/////////////////////////////////////////////////////////////////////////////
// Name:        src/common/dcbufcmn.cpp
// Purpose:     Buffered DC implementation
// Author:      Ron Lee, Jaakko Salli
// Modified by:
// Created:     Sep-20-2006
// RCS-ID:      $Id: dcbufcmn.cpp,v 1.4 2006/10/18 22:36:30 RD Exp $
// Copyright:   (c) wxWidgets team
// Licence:     wxWindows licence
/////////////////////////////////////////////////////////////////////////////

// ============================================================================
// declarations
// ============================================================================

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------

// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
#endif

#include "wx/dcbuffer.h"


// ============================================================================
// implementation
// ============================================================================

// ============================================================================
// wxSharedDCBufferManager
//   Helper class to free shared buffer when the app exists.
// ============================================================================

class wxSharedDCBufferManager
{
    friend class wxBufferedDC;
public:

    wxSharedDCBufferManager() { }
    ~wxSharedDCBufferManager() { }

    wxBitmap* GetBuffer(int w, int h)
    {
        if ( !m_buffer.IsOk() ||
             w > m_buffer.GetWidth() ||
             h > m_buffer.GetHeight() )
        {
            // Create slightly larger bitmap so we don't need to
            // be reallocating constantly when the user enlarges
            // the frame for the first time.
            m_buffer = wxBitmap(w, h);
        }

        return &m_buffer;
    }

private:
    wxBitmap    m_buffer;
};

static wxSharedDCBufferManager gs_sharedDCBufferManager;


// ============================================================================
// wxBufferedDC
// ============================================================================

void wxBufferedDC::UseBuffer(wxCoord w, wxCoord h)
{
    if ( !m_buffer )
    {
        if ( w == -1 || h == -1 )
            m_dc->GetSize(&w, &h);

        m_buffer = gs_sharedDCBufferManager.GetBuffer(w, h);
    }

    SelectObject(*m_buffer);
}

